@@ -22,8 +22,9 @@ func main() {
 	// Prepare agent gRPC handler
 	storkAgent := agent.NewStorkAgent(appMonitor)
 
-	// Prepare
+	// Prepare Prometheus exporters
 	promKeaExporter := agent.NewPromKeaExporter(appMonitor)
+	promBind9Exporter := agent.NewPromBind9Exporter(appMonitor)
 
 	// Prepare parse for command line flags.
 	parser := flags.NewParser(&storkAgent.Settings, flags.Default)
@@ -35,6 +36,11 @@ func main() {
 		log.Fatalf("FATAL error: %+v", err)
 	}
 
+	_, err = parser.AddGroup("Prometheus BIND 9 Exporter flags", "", &promBind9Exporter.Settings)
+	if err != nil {
+		log.Fatalf("FATAL error: %+v", err)
+	}
+
 	if _, err := parser.Parse(); err != nil {
 		code := 1
 		if fe, ok := err.(*flags.Error); ok {
@@ -48,5 +54,8 @@ func main() {
 	promKeaExporter.Start()
 	defer promKeaExporter.Shutdown()
 
+	promBind9Exporter.Start()
+	defer promBind9Exporter.Shutdown()
+
 	storkAgent.Serve()
 }
