@@ -1,8 +1,6 @@
 package dbmodel
 
 import (
-	"context"
-	"encoding/json"
 	"time"
 
 	"github.com/go-pg/pg/v9"
@@ -12,47 +10,15 @@ import (
 	dbops "isc.org/stork/server/database"
 )
 
-type Bind9Daemon struct {
-	Pid                int32
-	Name               string
-	Version            string
-	Active             bool
-	Uptime             int64
-	ReloadedAt         time.Time
-	ZoneCount          int64
-	AutomaticZoneCount int64
-	CacheHits          int64
-	CacheMisses        int64
-	CacheHitRatio      float64
-}
-
-type KeaDaemon struct {
-	Pid             int32
-	Name            string
-	Active          bool
-	Version         string
-	ExtendedVersion string
-	Config          *KeaConfig
-	Uptime          int64
-	ReloadedAt      time.Time
-}
-
-const AppTypeKea = "kea"
-
-type AppKea struct {
-	ExtendedVersion string
-	Daemons         []*KeaDaemon
-}
-
-const AppTypeBind9 = "bind9"
-
-type AppBind9 struct {
-	Daemon Bind9Daemon
-}
+const (
+	AppTypeKea   = "kea"
+	AppTypeBind9 = "bind9"
+)
 
 // Part of app table in database that describes metadata of app. In DB it is stored as JSONB.
 type AppMeta struct {
-	Version string
+	Version         string
+	ExtendedVersion string
 }
 
 // Represents an app held in app table in the database.
@@ -64,41 +30,10 @@ type App struct {
 	Type      string // currently supported types are: "kea" and "bind9"
 	Active    bool
 	Meta      AppMeta
-	Details   interface{} // here we have either AppKea or AppBind9
 
 	AccessPoints []*AccessPoint
-}
 
-// This is a hook to go-pg that is called just after reading rows from database.
-// It reconverts app details from json string maps to particular Go structure.
-func (app *App) AfterScan(ctx context.Context) error {
-	if app.Details == nil {
-		return nil
-	}
-
-	bytes, err := json.Marshal(app.Details)
-	if err != nil {
-		return errors.Wrapf(err, "problem with marshaling %s app details: %v ", app.Type, app.Details)
-	}
-
-	switch app.Type {
-	case AppTypeKea:
-		var keaDetails AppKea
-		err = json.Unmarshal(bytes, &keaDetails)
-		if err != nil {
-			return errors.Wrapf(err, "problem with unmarshaling kea app details")
-		}
-		app.Details = keaDetails
-
-	case AppTypeBind9:
-		var bind9Details AppBind9
-		err = json.Unmarshal(bytes, &bind9Details)
-		if err != nil {
-			return errors.Wrapf(err, "problem with unmarshaling BIND 9 app details")
-		}
-		app.Details = bind9Details
-	}
-	return nil
+	Daemons []*Daemon
 }
 
 // updateAppAccessPoints updates the associated application access points into
@@ -136,6 +71,86 @@ func updateAppAccessPoints(tx *pg.Tx, app *App, update bool) (err error) {
 	return nil
 }
 
+// Adds or updates daemons for an app. If any daemons already exist for the app,
+// they are removed and the new daemons will be added instead.
+func updateAppDaemons(tx *pg.Tx, app *App) (err error) {
+	// Delete the existing daemons, because the updated app may have different
+	// set of daemons. In particular, some of them may be gone.
+	ids := []int64{}
+	for _, d := range app.Daemons {
+		if d.ID > 0 {
+			ids = append(ids, d.ID)
+		}
+	}
+	q := tx.Model((*Daemon)(nil)).
+		Where("daemon.app_id = ?", app.ID)
+	if len(ids) > 0 {
+		q = q.Where("daemon.id NOT IN (?)", pg.In(ids))
+	}
+	_, err = q.Delete()
+	if err != nil {
+		return errors.Wrapf(err, "problem with deleting daemons for an updated app %d", app.ID)
+	}
+
+	// Add updated daemons.
+	for i := range app.Daemons {
+		daemon := app.Daemons[i]
+
+		// Make sure the inserted daemon references the app.
+		daemon.AppID = app.ID
+		if daemon.ID == 0 {
+			// Add the new entry to the daemon table.
+			_, err = tx.Model(daemon).Insert()
+		} else {
+			_, err = tx.Model(daemon).WherePK().Update()
+		}
+		if err != nil {
+			return errors.Wrapf(err, "problem with upserting daemon to app %d: %v", app.ID, daemon)
+		}
+
+		if daemon.KeaDaemon != nil {
+			// Make sure that the kea_daemon references the daemon.
+			daemon.KeaDaemon.DaemonID = daemon.ID
+			if daemon.KeaDaemon.ID == 0 {
+				_, err = tx.Model(daemon.KeaDaemon).Insert()
+			} else {
+				_, err = tx.Model(daemon.KeaDaemon).WherePK().Update()
+			}
+			if err != nil {
+				return errors.Wrapf(err, "problem with upserting Kea daemon to app %d: %v",
+					app.ID, daemon.KeaDaemon)
+			}
+
+			if daemon.KeaDaemon.KeaDHCPDaemon != nil {
+				// Make sure that the kea_dhcp_daemon references the kea_daemon.
+				daemon.KeaDaemon.KeaDHCPDaemon.KeaDaemonID = daemon.KeaDaemon.ID
+				if daemon.KeaDaemon.KeaDHCPDaemon.ID == 0 {
+					_, err = tx.Model(daemon.KeaDaemon.KeaDHCPDaemon).Insert()
+				} else {
+					_, err = tx.Model(daemon.KeaDaemon.KeaDHCPDaemon).WherePK().Update()
+				}
+				if err != nil {
+					return errors.Wrapf(err, "problem with upserting Kea DHCP daemon to app %d: %v",
+						app.ID, daemon.KeaDaemon.KeaDHCPDaemon)
+				}
+			}
+		} else if daemon.Bind9Daemon != nil {
+			// Make sure that the bind9_daemon references the daemon.
+			daemon.Bind9Daemon.DaemonID = daemon.ID
+			if daemon.Bind9Daemon.ID == 0 {
+				_, err = tx.Model(daemon.Bind9Daemon).Insert()
+			} else {
+				_, err = tx.Model(daemon.Bind9Daemon).WherePK().Update()
+			}
+			if err != nil {
+				return errors.Wrapf(err, "problem with upserting BIND9 daemon to app %d: %v",
+					app.ID, daemon.Bind9Daemon)
+			}
+		}
+	}
+	return nil
+}
+
 // Adds application into the database. The dbIface object may either be a pg.DB
 // object or pg.Tx. In the latter case this function uses existing transaction
 // to add an app.
@@ -154,6 +169,11 @@ func AddApp(dbIface interface{}, app *App) error {
 		return errors.Wrapf(err, "problem with inserting app %v", app)
 	}
 
+	err = updateAppDaemons(tx, app)
+	if err != nil {
+		return errors.WithMessagef(err, "problem with inserting daemons for a new app")
+	}
+
 	// Add access points.
 	err = updateAppAccessPoints(tx, app, false)
 	if err != nil {
@@ -184,6 +204,11 @@ func UpdateApp(dbIface interface{}, app *App) error {
 		return errors.Wrapf(err, "problem with updating app %v", app)
 	}
 
+	err = updateAppDaemons(tx, app)
+	if err != nil {
+		return errors.WithMessagef(err, "problem with updating daemons for app %d", app.ID)
+	}
+
 	// Update access points.
 	err = updateAppAccessPoints(tx, app, true)
 	if err != nil {
@@ -200,6 +225,8 @@ func GetAppByID(db *pg.DB, id int64) (*App, error) {
 	q := db.Model(&app)
 	q = q.Relation("Machine")
 	q = q.Relation("AccessPoints")
+	q = q.Relation("Daemons.KeaDaemon.KeaDHCPDaemon")
+	q = q.Relation("Daemons.Bind9Daemon")
 	q = q.Where("app.id = ?", id)
 	err := q.Select()
 	if err == pg.ErrNoRows {
@@ -215,7 +242,10 @@ func GetAppsByMachine(db *pg.DB, machineID int64) ([]App, error) {
 
 	q := db.Model(&apps)
 	q = q.Relation("AccessPoints")
+	q = q.Relation("Daemons.KeaDaemon.KeaDHCPDaemon")
+	q = q.Relation("Daemons.Bind9Daemon")
 	q = q.Where("machine_id = ?", machineID)
+	q = q.OrderExpr("id ASC")
 	err := q.Select()
 	if err != nil {
 		return nil, errors.Wrapf(err, "problem with getting apps")
@@ -223,7 +253,7 @@ func GetAppsByMachine(db *pg.DB, machineID int64) ([]App, error) {
 	return apps, nil
 }
 
-// Fetches all app by type from the database.
+// Fetches all apps by type including the corresponding services.
 func GetAppsByType(db *pg.DB, appType string) ([]App, error) {
 	var apps []App
 
@@ -231,6 +261,16 @@ func GetAppsByType(db *pg.DB, appType string) ([]App, error) {
 	q = q.Where("type = ?", appType)
 	q = q.Relation("Machine")
 	q = q.Relation("AccessPoints")
+
+	switch appType {
+	case AppTypeKea:
+		q = q.Relation("Daemons.Services.HAService")
+		q = q.Relation("Daemons.KeaDaemon.KeaDHCPDaemon")
+	case AppTypeBind9:
+		q = q.Relation("Daemons.Bind9Daemon")
+	}
+
+	q = q.OrderExpr("id ASC")
 	err := q.Select()
 	if err != nil {
 		return nil, errors.Wrapf(err, "problem with getting %s apps", appType)
@@ -238,10 +278,14 @@ func GetAppsByType(db *pg.DB, appType string) ([]App, error) {
 	return apps, nil
 }
 
-// Fetches a collection of apps from the database. The offset and limit specify the
-// beginning of the page and the maximum size of the page. Limit has to be greater
-// then 0, otherwise error is returned.
-func GetAppsByPage(db *pg.DB, offset int64, limit int64, text string, appType string) ([]App, int64, error) {
+// Fetches a collection of apps from the database. The offset and
+// limit specify the beginning of the page and the maximum size of the
+// page. Limit has to be greater then 0, otherwise error is
+// returned. sortField allows indicating sort column in database and
+// sortDir allows selection the order of sorting. If sortField is
+// empty then id is used for sorting.  in SortDirAny is used then ASC
+// order is used.
+func GetAppsByPage(db *pg.DB, offset int64, limit int64, filterText *string, appType string, sortField string, sortDir SortDirEnum) ([]App, int64, error) {
 	if limit == 0 {
 		return nil, 0, errors.New("limit should be greater than 0")
 	}
@@ -251,32 +295,53 @@ func GetAppsByPage(db *pg.DB, offset int64, limit int64, text string, appType st
 	q := db.Model(&apps)
 	q = q.Relation("AccessPoints")
 	q = q.Relation("Machine")
+	q = q.Relation("Daemons.KeaDaemon.KeaDHCPDaemon")
+	q = q.Relation("Daemons.Bind9Daemon")
 	if appType != "" {
 		q = q.Where("type = ?", appType)
 	}
-	if text != "" {
-		text = "%" + text + "%"
+	if filterText != nil {
+		text := "%" + *filterText + "%"
 		q = q.WhereGroup(func(qq *orm.Query) (*orm.Query, error) {
+			qq = qq.WhereOr("type ILIKE ?", text)
 			qq = qq.WhereOr("meta->>'Version' ILIKE ?", text)
+			qq = qq.WhereOr("machine.address ILIKE ?", text)
+			qq = qq.WhereOr("machine.state->>'Hostname' ILIKE ?", text)
 			return qq, nil
 		})
 	}
 
-	// and then, first get total count
-	total, err := q.Clone().Count()
-	if err != nil {
-		return nil, 0, errors.Wrapf(err, "problem with getting apps total")
-	}
+	// prepare sorting expression, offset and limit
+	ordExpr := prepareOrderExpr("app", sortField, sortDir)
+	q = q.OrderExpr(ordExpr)
+	q = q.Offset(int(offset))
+	q = q.Limit(int(limit))
 
-	// then retrieve given page of rows
-	q = q.Order("id ASC").Offset(int(offset)).Limit(int(limit))
-	err = q.Select()
+	total, err := q.SelectAndCount()
 	if err != nil {
 		return nil, 0, errors.Wrapf(err, "problem with getting apps")
 	}
 	return apps, int64(total), nil
 }
 
+func GetAllApps(db *pg.DB) ([]App, error) {
+	var apps []App
+
+	// prepare query
+	q := db.Model(&apps)
+	q = q.Relation("AccessPoints")
+	q = q.Relation("Daemons.KeaDaemon.KeaDHCPDaemon")
+	q = q.Relation("Daemons.Bind9Daemon")
+	q = q.OrderExpr("id ASC")
+
+	// retrieve apps from db
+	err := q.Select()
+	if err != nil {
+		return nil, errors.Wrapf(err, "problem with getting apps")
+	}
+	return apps, nil
+}
+
 func DeleteApp(db *pg.DB, app *App) error {
 	err := db.Delete(app)
 	if err != nil {
@@ -285,45 +350,33 @@ func DeleteApp(db *pg.DB, app *App) error {
 	return nil
 }
 
-func GetAllApps(db *pg.DB) ([]App, error) {
-	var apps []App
-
-	// prepare query
-	q := db.Model(&apps)
-	q = q.Relation("AccessPoints")
-
-	// retrieve apps from db
-	err := q.Select()
-	if err != nil {
-		return nil, errors.Wrapf(err, "problem with getting apps")
-	}
-	return apps, nil
-}
-
 // Returns a list of names of active DHCP deamons. This is useful for
 // creating commands to be send to active DHCP servers.
-func (app *App) GetActiveDHCPDaemonNames() (deamons []string) {
-	if kea, ok := app.Details.(AppKea); ok {
-		for _, d := range kea.Daemons {
-			if d.Active && (d.Name == "dhcp4" || d.Name == "dhcp6") {
-				deamons = append(deamons, d.Name)
-			}
+func (app *App) GetActiveDHCPDaemonNames() (daemons []string) {
+	if app.Type != AppTypeKea {
+		return daemons
+	}
+	for _, d := range app.Daemons {
+		if d.Active && (d.Name == DaemonNameDHCPv4 || d.Name == DaemonNameDHCPv6) {
+			daemons = append(daemons, d.Name)
 		}
 	}
-	return deamons
+	return daemons
 }
 
 // Returns local subnet ID for a given subnet prefix. It iterates over the
 // daemons' configurations and searches for a subnet with matching prefix.
 // If the match isn't found, the value of 0 is returned.
 func (app *App) GetLocalSubnetID(prefix string) int64 {
-	if kea, ok := app.Details.(AppKea); ok {
-		for _, d := range kea.Daemons {
-			if d.Config != nil {
-				if id := d.Config.GetLocalSubnetID(prefix); id > 0 {
-					return id
-				}
-			}
+	if app.Type != AppTypeKea {
+		return 0
+	}
+	for _, d := range app.Daemons {
+		if d.KeaDaemon == nil || d.KeaDaemon.Config == nil {
+			continue
+		}
+		if id := d.KeaDaemon.Config.GetLocalSubnetID(prefix); id > 0 {
+			return id
 		}
 	}
 	return 0
