@@ -47,6 +47,12 @@ type Host struct {
 	IPReservations  []IPReservation
 
 	LocalHosts []LocalHost
+
+	// This flag is used to indicate that some changes have been applied to
+	//the Host instance locally and that these changes should be applied in
+	// the database too. It also indicates that the new app should be
+	// associated with the host upon the call to the CommitSubnetHostsIntoDB.
+	UpdateOnCommit bool `pg:"-"`
 }
 
 // This structure links a host entry stored in the database with an app from
@@ -277,7 +283,7 @@ func GetHostsBySubnetID(dbIface interface{}, subnetID int64) ([]Host, error) {
 	}
 	defer rollback()
 
-	err = tx.Model(&hosts).
+	q := tx.Model(&hosts).
 		Relation("HostIdentifiers", func(q *orm.Query) (*orm.Query, error) {
 			return q.Order("host_identifier.id ASC"), nil
 		}).
@@ -285,10 +291,20 @@ func GetHostsBySubnetID(dbIface interface{}, subnetID int64) ([]Host, error) {
 			return q.Order("ip_reservation.id ASC"), nil
 		}).
 		Relation("LocalHosts").
-		Where("host.subnet_id = ?", subnetID).
-		OrderExpr("id ASC").
-		Select()
+		OrderExpr("id ASC")
 
+	// Subnet ID is never zero, it may be NULL. The reason for it is that we
+	// have a foreign key that requires subnet to exist for non NULL value.
+	// This constraint allows for NULL subnet_id though. Therefore, searching
+	// for a host with subnet_id of zero is really searching for a host with
+	// the NULL value.
+	if subnetID == 0 {
+		q = q.Where("host.subnet_id IS NULL")
+	} else {
+		q = q.Where("host.subnet_id = ?", subnetID)
+	}
+
+	err = q.Select()
 	if err != nil {
 		if err == pg.ErrNoRows {
 			return nil, nil
@@ -299,17 +315,37 @@ func GetHostsBySubnetID(dbIface interface{}, subnetID int64) ([]Host, error) {
 	return hosts, err
 }
 
-// Fetches a collection of hosts from the database. The offset and limit
-// specify the beginning of the page and the maximum size of the page. The
-// optional subnetID is used to fetch hosts belonging to the particular
-// IPv4 or IPv6 subnet. If this value is set to nil all subnets are returned.
-// The value of 0 indicates that only global hosts are to be returned.
-// Filtering text allows for searching hosts by reserved IP addresses
-// and/or host identifiers specified using hexadecimal digits. It is
-// allowed to specify colons while searching by hosts by host identifiers.
-func GetHostsByPage(db *pg.DB, offset, limit int64, subnetID *int64, filterText *string) ([]Host, int64, error) {
+// Fetches a collection of hosts from the database. The offset and
+// limit specify the beginning of the page and the maximum size of the
+// page. The appID, if different than 0, is used to fetch hosts whose
+// local hosts belong to indicated app. The optional subnetID is used
+// to fetch hosts belonging to the particular IPv4 or IPv6 subnet. If
+// this value is set to nil all subnets are returned.  The value of 0
+// indicates that only global hosts are to be returned. Filtering text
+// allows for searching hosts by reserved IP addresses and/or host
+// identifiers specified using hexadecimal digits. It is allowed to
+// specify colons while searching by hosts by host
+// identifiers. sortField allows indicating sort column in database
+// and sortDir allows selection the order of sorting. If sortField is
+// empty then id is used for sorting.  in SortDirAny is used then ASC
+// order is used.
+func GetHostsByPage(db *pg.DB, offset, limit int64, appID int64, subnetID *int64, filterText *string, sortField string, sortDir SortDirEnum) ([]Host, int64, error) {
 	hosts := []Host{}
-	q := db.Model(&hosts).DistinctOn("host.id")
+	q := db.Model(&hosts)
+
+	// prepare distinct on expression to include sort field, otherwise distinct on will fail
+	distingOnFields := "host.id"
+	if sortField != "" && sortField != "id" && sortField != "host.id" {
+		distingOnFields = sortField + ", " + distingOnFields
+	}
+	q = q.DistinctOn(distingOnFields)
+
+	// When filtering by appID we also need the local_host table as it holds the
+	// application identifier.
+	if appID != 0 {
+		q = q.Join("INNER JOIN local_host AS lh ON host.id = lh.host_id")
+		q = q.Where("lh.app_id = ?", appID)
+	}
 
 	if subnetID != nil {
 		if *subnetID == 0 {
@@ -333,6 +369,13 @@ func GetHostsByPage(db *pg.DB, offset, limit int64, subnetID *int64, filterText
 			q = q.WhereOr("text(r.address) LIKE ?", "%"+*filterText+"%").
 				WhereOr("i.type::text LIKE ?", "%"+*filterText+"%").
 				WhereOr("encode(i.value, 'hex') LIKE ?", "%"+colonlessFilterText+"%")
+			// If someone typed in "global" in the search box we have to search for all
+			// global reservations, i.e. those that have subnet ID equal to NULL.
+			// We don't include this condition if it has been already included earlier
+			// in this function for subnetID parameter equal to 0.
+			if strings.HasPrefix(string("global"), *filterText) && (subnetID == nil || *subnetID != 0) {
+				q = q.WhereOr("host.subnet_id IS NULL")
+			}
 			return q, nil
 		})
 	}
@@ -344,7 +387,9 @@ func GetHostsByPage(db *pg.DB, offset, limit int64, subnetID *int64, filterText
 		Relation("IPReservations", func(q *orm.Query) (*orm.Query, error) {
 			return q.Order("ip_reservation.id ASC"), nil
 		}).
-		Relation("LocalHosts")
+		Relation("LocalHosts").
+		Relation("LocalHosts.App").
+		Relation("LocalHosts.App.AccessPoints")
 
 	// Only join the subnet if querying all hosts or hosts belonging to a
 	// given subnet.
@@ -352,10 +397,11 @@ func GetHostsByPage(db *pg.DB, offset, limit int64, subnetID *int64, filterText
 		q = q.Relation("Subnet")
 	}
 
-	q = q.
-		OrderExpr("host.id ASC").
-		Offset(int(offset)).
-		Limit(int(limit))
+	// prepare sorting expression, offser and limit
+	ordExpr := prepareOrderExpr("host", sortField, sortDir)
+	q = q.OrderExpr(ordExpr)
+	q = q.Offset(int(offset))
+	q = q.Limit(int(limit))
 
 	total, err := q.SelectAndCount()
 	if err != nil {
@@ -442,35 +488,48 @@ func AddAppToHost(dbIface interface{}, host *Host, app *App, source string, seq
 	return err
 }
 
-// Iterates over the hosts belonging to the given subnet and stores them
-// or updates in the database.
-func CommitSubnetHostsIntoDB(tx *pg.Tx, subnet *Subnet, app *App, source string, seq int64) (err error) {
-	for i := range subnet.Hosts {
-		// Make sure the host associated with the current subnet.
-		subnet.Hosts[i].SubnetID = subnet.ID
-		if subnet.Hosts[i].ID == 0 {
-			err = AddHost(tx, &subnet.Hosts[i])
+// Iterates over the list of hosts and commits them into the database. The hosts
+// can be associated with a subnet or can be made global.
+func commitHostsIntoDB(tx *pg.Tx, hosts []Host, subnetID int64, app *App, source string, seq int64) (err error) {
+	for i := range hosts {
+		hosts[i].SubnetID = subnetID
+		newHost := (hosts[i].ID == 0)
+		if newHost {
+			err = AddHost(tx, &hosts[i])
 			if err != nil {
 				err = errors.WithMessagef(err, "unable to add detected host to the database")
 				return err
 			}
-		} else {
-			err = UpdateHost(tx, &subnet.Hosts[i])
+		} else if hosts[i].UpdateOnCommit {
+			err = UpdateHost(tx, &hosts[i])
 			if err != nil {
 				err = errors.WithMessagef(err, "unable to update detected host in the database")
 				return err
 			}
 		}
-		err = AddAppToHost(tx, &subnet.Hosts[i], app, source, seq)
-		if err != nil {
-			err = errors.WithMessagef(err, "unable to associate detected host with Kea app having id %d",
-				app.ID)
-			return err
+		if newHost || hosts[i].UpdateOnCommit {
+			err = AddAppToHost(tx, &hosts[i], app, source, seq)
+			if err != nil {
+				err = errors.WithMessagef(err, "unable to associate detected host with Kea app having id %d",
+					app.ID)
+				return err
+			}
 		}
 	}
 	return nil
 }
 
+// Iterates over the list of hosts and commits them as global hosts.
+func CommitGlobalHostsIntoDB(tx *pg.Tx, hosts []Host, app *App, source string, seq int64) (err error) {
+	return commitHostsIntoDB(tx, hosts, 0, app, source, seq)
+}
+
+// Iterates over the hosts belonging to the given subnet and stores them
+// or updates in the database.
+func CommitSubnetHostsIntoDB(tx *pg.Tx, subnet *Subnet, app *App, source string, seq int64) (err error) {
+	return commitHostsIntoDB(tx, subnet.Hosts, subnet.ID, app, source, seq)
+}
+
 // This function returns next value of the bulk_update_seq. The returned value
 // is used during bulk updates of data within a database. For example, when
 // inserting or updating many host reservations, sequence value for all host
