@@ -49,6 +49,9 @@ type Subnet struct {
 	LocalSubnets []*LocalSubnet
 
 	Hosts []Host
+
+	AddrUtilization int16
+	PdUtilization   int16
 }
 
 // Hook executed after inserting a subnet to the database. It updates subnet
@@ -316,17 +319,21 @@ func GetAllSubnets(db *pg.DB, family int) ([]Subnet, error) {
 	return subnets, err
 }
 
-// Fetches a collection of subnets from the database. The offset and limit
-// specify the beginning of the page and the maximum size of the page. The appID
-// is used to filter subnets to those handled by the given application.
-// The family is used to filter by IPv4 (if 4) or IPv6 (if 6). For all other values
-// of the family parameter both IPv4 and IPv6 subnets are returned. The filterText
-// can be used to match the subnet prefix or pool ranges. The nil value disables
-// such filtering. This function returns a collection of subnets, the total number
-// of subnets and error.
-func GetSubnetsByPage(db *pg.DB, offset, limit, appID, family int64, filterText *string) ([]Subnet, int64, error) {
+// Fetches a collection of subnets from the database. The offset and
+// limit specify the beginning of the page and the maximum size of the
+// page. The appID is used to filter subnets to those handled by the
+// given application.  The family is used to filter by IPv4 (if 4) or
+// IPv6 (if 6). For all other values of the family parameter both IPv4
+// and IPv6 subnets are returned. The filterText can be used to match
+// the subnet prefix or pool ranges. The nil value disables such
+// filtering. sortField allows indicating sort column in database and
+// sortDir allows selection the order of sorting. If sortField is
+// empty then id is used for sorting.  in SortDirAny is used then ASC
+// order is used. This function returns a collection of subnets, the
+// total number of subnets and error.
+func GetSubnetsByPage(db *pg.DB, offset, limit, appID, family int64, filterText *string, sortField string, sortDir SortDirEnum) ([]Subnet, int64, error) {
 	subnets := []Subnet{}
-	q := db.Model(&subnets).DistinctOn("subnet.id")
+	q := db.Model(&subnets)
 
 	// When filtering by appID we also need the local_subnet table as it holds the
 	// application identifier.
@@ -346,7 +353,8 @@ func GetSubnetsByPage(db *pg.DB, offset, limit, appID, family int64, filterText
 			return q.Order("prefix_pool.id ASC"), nil
 		}).
 		Relation("SharedNetwork").
-		Relation("LocalSubnets.App.AccessPoints")
+		Relation("LocalSubnets.App.AccessPoints").
+		Relation("LocalSubnets.App.Machine")
 
 	// Let's be liberal and allow other values than 0 too. The only special
 	// ones are 4 and 6.
@@ -372,9 +380,11 @@ func GetSubnetsByPage(db *pg.DB, offset, limit, appID, family int64, filterText
 		})
 	}
 
-	q = q.OrderExpr("subnet.id ASC").
-		Offset(int(offset)).
-		Limit(int(limit))
+	// prepare sorting expression, offser and limit
+	ordExpr := prepareOrderExpr("subnet", sortField, sortDir)
+	q = q.OrderExpr(ordExpr)
+	q = q.Offset(int(offset))
+	q = q.Limit(int(limit))
 
 	// This returns the limited results plus the total number of records.
 	total, err := q.SelectAndCount()
@@ -387,6 +397,26 @@ func GetSubnetsByPage(db *pg.DB, offset, limit, appID, family int64, filterText
 	return subnets, int64(total), err
 }
 
+// Get list of Subnets with LocalSubnets ordered by SharedNetworkID
+func GetSubnetsWithLocalSubnets(db *pg.DB) ([]*Subnet, error) {
+	subnets := []*Subnet{}
+	q := db.Model(&subnets)
+	// only selected columns are returned for performance reasons
+	q = q.Column("id", "shared_network_id", "prefix")
+	q = q.Relation("LocalSubnets")
+	q = q.Order("shared_network_id ASC")
+
+	err := q.Select()
+	if err != nil {
+		if err == pg.ErrNoRows {
+			return nil, nil
+		}
+		err = errors.Wrap(err, "problem with getting all subnets")
+		return nil, err
+	}
+	return subnets, nil
+}
+
 // Associates an application with the subnet having a specified ID and prefix.
 // Internally, the association is made via the local_subnet table which holds
 // information about the subnet from the given app perspective, local subnet
@@ -438,7 +468,7 @@ func AddAppToSubnet(dbIface interface{}, subnet *Subnet, app *App) error {
 
 // Dissociates an application from the subnet having a specified id.
 // The first returned value indicates if any row was removed from the
-// app_to_service table.
+// daemon_to_service table.
 func DeleteAppFromSubnet(db *pg.DB, subnetID int64, appID int64) (bool, error) {
 	localSubnet := &LocalSubnet{
 		AppID:    appID,
@@ -568,3 +598,18 @@ func (lsn *LocalSubnet) UpdateStats(db *pg.DB, stats map[string]interface{}) err
 	}
 	return err
 }
+
+// Update utilization in Subnet.
+func (s *Subnet) UpdateUtilization(db *pg.DB, addrUtilization, pdUtilization int16) error {
+	s.AddrUtilization = addrUtilization
+	s.PdUtilization = pdUtilization
+	q := db.Model(s)
+	q = q.Column("addr_utilization", "pd_utilization")
+	q = q.WherePK()
+	_, err := q.Update()
+	if err != nil {
+		err = errors.Wrapf(err, "problem with updating utilization in the subnet: %d",
+			s.ID)
+	}
+	return err
+}
