@@ -28,6 +28,9 @@ import { ProgressSpinnerModule } from 'primeng/progressspinner'
 import { TooltipModule } from 'primeng/tooltip'
 import { PasswordModule } from 'primeng/password'
 import { CardModule } from 'primeng/card'
+import { SplitButtonModule } from 'primeng/splitbutton'
+import { FieldsetModule } from 'primeng/fieldset'
+import { OverlayPanelModule } from 'primeng/overlaypanel'
 
 // Generated API modules
 import { ApiModule, BASE_PATH, Configuration, ConfigurationParameters } from './backend'
@@ -57,6 +60,9 @@ import { SubnetsPageComponent } from './subnets-page/subnets-page.component'
 import { SharedNetworksPageComponent } from './shared-networks-page/shared-networks-page.component'
 import { SubnetBarComponent } from './subnet-bar/subnet-bar.component'
 import { HostsPageComponent } from './hosts-page/hosts-page.component'
+import { SettingsPageComponent } from './settings-page/settings-page.component'
+import { HelpTipComponent } from './help-tip/help-tip.component'
+import { GlobalSearchComponent } from './global-search/global-search.component'
 
 export function cfgFactory() {
     const params: ConfigurationParameters = {
@@ -97,12 +103,14 @@ export function preinitApp(appInitService: AppInitService) {
         SharedNetworksPageComponent,
         SubnetBarComponent,
         HostsPageComponent,
+        SettingsPageComponent,
+        HelpTipComponent,
+        GlobalSearchComponent,
     ],
     imports: [
         BrowserModule,
         BrowserAnimationsModule,
         HttpClientModule,
-        FormsModule,
         AppRoutingModule,
         FormsModule,
         ReactiveFormsModule,
@@ -129,6 +137,9 @@ export function preinitApp(appInitService: AppInitService) {
         TooltipModule,
         PasswordModule,
         CardModule,
+        SplitButtonModule,
+        FieldsetModule,
+        OverlayPanelModule,
     ],
     providers: [
         {
